@@ -29,6 +29,7 @@
 #include <stdlib.h>
 
 #include "glib.h"
+#include "gprintfint.h"
 
 #ifdef G_PLATFORM_WIN32
 #define STRICT
@@ -102,7 +103,8 @@ extern const char **_g_charset_get_aliases (const char *canonical_name);
  * GLib provides g_convert() and g_locale_to_utf8() which are likely
  * more convenient than the raw iconv wrappers.
  * 
- * Return value: a "conversion descriptor"
+ * Return value: a "conversion descriptor", or (GIConv)-1 if
+ *  opening the converter failed.
  **/
 GIConv
 g_iconv_open (const gchar  *to_codeset,
@@ -121,20 +123,21 @@ g_iconv_open (const gchar  *to_codeset,
 	  while (*p)
 	    {
 	      if (try_conversion (to_codeset, *p, &cd))
-		return (GIConv)cd;
+		goto out;
 
 	      if (try_to_aliases (to_aliases, *p, &cd))
-		return (GIConv)cd;
+		goto out;
 
 	      p++;
 	    }
 	}
 
       if (try_to_aliases (to_aliases, from_codeset, &cd))
-	return (GIConv)cd;
+	goto out;
     }
 
-  return (GIConv)cd;
+ out:
+  return (cd == (iconv_t)-1) ? (GIConv)-1 : (GIConv)cd;
 }
 
 /**
@@ -336,7 +339,7 @@ open_converter (const gchar *to_codeset,
   
   /* create our key */
   key = g_alloca (strlen (from_codeset) + strlen (to_codeset) + 2);
-  sprintf (key, "%s:%s", from_codeset, to_codeset);
+  _g_sprintf (key, "%s:%s", from_codeset, to_codeset);
   
   G_LOCK (iconv_cache_lock);
   
@@ -358,9 +361,9 @@ open_converter (const gchar *to_codeset,
 	   * NULL for anything but inbuf; work around that. (NULL outbuf
 	   * or NULL *outbuf is allowed by Unix98.)
 	   */
-	  gint inbytes_left = 0;
+	  gsize inbytes_left = 0;
 	  gchar *outbuf = NULL;
-	  gint outbytes_left = 0;
+	  gsize outbytes_left = 0;
 		
           cd = bucket->cd;
           bucket->used = TRUE;
@@ -879,7 +882,7 @@ strdup_len (const gchar *string,
 {
   gsize real_len;
 
-  if (!g_utf8_validate (string, -1, NULL))
+  if (!g_utf8_validate (string, len, NULL))
     {
       if (bytes_read)
 	*bytes_read = 0;
