@@ -273,44 +273,6 @@ utf8_str (const gchar *utf8,
   return buf;
 }
 
-static gboolean
-str_has_suffix (const gchar  *str,
-		const gchar  *suffix)
-{
-  int str_len;
-  int suffix_len;
-  
-  g_return_val_if_fail (str != NULL, FALSE);
-  g_return_val_if_fail (suffix != NULL, FALSE);
-
-  str_len = strlen (str);
-  suffix_len = strlen (suffix);
-
-  if (str_len < suffix_len)
-    return FALSE;
-
-  return strcmp (str + str_len - suffix_len, suffix) == 0;
-}
-
-static gboolean
-str_has_prefix (const gchar  *str,
-		const gchar  *prefix)
-{
-  int str_len;
-  int prefix_len;
-  
-  g_return_val_if_fail (str != NULL, FALSE);
-  g_return_val_if_fail (prefix != NULL, FALSE);
-
-  str_len = strlen (str);
-  prefix_len = strlen (prefix);
-
-  if (str_len < prefix_len)
-    return FALSE;
-  
-  return strncmp (str, prefix, prefix_len) == 0;
-}
-
 static void
 set_unescape_error (GMarkupParseContext *context,
                     GError             **error,
@@ -367,6 +329,7 @@ unescape_text (GMarkupParseContext *context,
                gchar              **unescaped,
                GError             **error)
 {
+#define MAX_ENT_LEN 5
   GString *str;
   const gchar *p;
   UnescapeState state;
@@ -449,6 +412,11 @@ unescape_text (GMarkupParseContext *context,
 
         case USTATE_INSIDE_ENTITY_NAME:
           {
+            gchar buf[MAX_ENT_LEN+1] = {
+              '\0', '\0', '\0', '\0', '\0', '\0'
+            };
+            gchar *dest;
+
             while (p != text_end)
               {
                 if (*p == ';')
@@ -473,22 +441,31 @@ unescape_text (GMarkupParseContext *context,
               {
                 if (p != text_end)
                   {
-		    gchar *ent = g_strndup (start, p - start);
+                    const gchar *src;
+                
+                    src = start;
+                    dest = buf;
+                    while (src != p)
+                      {
+                        *dest = *src;
+                        ++dest;
+                        ++src;
+                      }
 
                     /* move to after semicolon */
                     p = g_utf8_next_char (p);
                     start = p;
                     state = USTATE_INSIDE_TEXT;
 
-                    if (strcmp (ent, "lt") == 0)
+                    if (strcmp (buf, "lt") == 0)
                       g_string_append_c (str, '<');
-                    else if (strcmp (ent, "gt") == 0)
+                    else if (strcmp (buf, "gt") == 0)
                       g_string_append_c (str, '>');
-                    else if (strcmp (ent, "amp") == 0)
+                    else if (strcmp (buf, "amp") == 0)
                       g_string_append_c (str, '&');
-                    else if (strcmp (ent, "quot") == 0)
+                    else if (strcmp (buf, "quot") == 0)
                       g_string_append_c (str, '"');
-                    else if (strcmp (ent, "apos") == 0)
+                    else if (strcmp (buf, "apos") == 0)
                       g_string_append_c (str, '\'');
                     else
                       {
@@ -496,9 +473,8 @@ unescape_text (GMarkupParseContext *context,
                                             p, text_end,
                                             G_MARKUP_ERROR_PARSE,
                                             _("Entity name '%s' is not known"),
-                                            ent);
+                                            buf);
                       }
-		    g_free (ent);
                   }
                 else
                   {
@@ -652,6 +628,8 @@ unescape_text (GMarkupParseContext *context,
       *unescaped = g_string_free (str, FALSE);
       return TRUE;
     }
+
+#undef MAX_ENT_LEN
 }
 
 static gboolean
@@ -749,6 +727,8 @@ find_current_text_end (GMarkupParseContext *context)
 
   while (next && *next)
     {
+      if (p == next)
+	next++;
       p = next;
       next = g_utf8_find_next_char (p, end);
     }
@@ -1531,13 +1511,13 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
 		  context->balance--;
 		  add_to_partial (context, context->start, context->iter);
 		  context->start = context->iter;
-		  if ((str_has_prefix (context->partial_chunk->str, "<?")
-		       && str_has_suffix (context->partial_chunk->str, "?")) ||
-		      (str_has_prefix (context->partial_chunk->str, "<!--")
-		       && str_has_suffix (context->partial_chunk->str, "--")) ||
-		      (str_has_prefix (context->partial_chunk->str, "<![CDATA[") 
-		       && str_has_suffix (context->partial_chunk->str, "]]")) ||
-		      (str_has_prefix (context->partial_chunk->str, "<!DOCTYPE")
+		  if ((g_str_has_prefix (context->partial_chunk->str, "<?")
+		       && g_str_has_suffix (context->partial_chunk->str, "?")) ||
+		      (g_str_has_prefix (context->partial_chunk->str, "<!--")
+		       && g_str_has_suffix (context->partial_chunk->str, "--")) ||
+		      (g_str_has_prefix (context->partial_chunk->str, "<![CDATA[") 
+		       && g_str_has_suffix (context->partial_chunk->str, "]]")) ||
+		      (g_str_has_prefix (context->partial_chunk->str, "<!DOCTYPE")
 		       && context->balance == 0)) 
 		    break;
 		}
@@ -1563,25 +1543,13 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               advance_char (context); /* advance past close angle */
               add_to_partial (context, context->start, context->iter);
 
-	      if (str_has_prefix (context->partial_chunk->str, "<![CDATA[")) 
-		{
-		  if (context->parser->text)
-		    (*context->parser->text) (context,
-					      context->partial_chunk->str + 9,
-					      context->partial_chunk->len - 12,
-					      context->user_data,
-					      &tmp_error);
-		}
-	      else 
-		{
-		  if (context->parser->passthrough)
-		    (*context->parser->passthrough) (context,
-						     context->partial_chunk->str,
-						     context->partial_chunk->len,
-						     context->user_data,
-						     &tmp_error);
-		}
-
+              if (context->parser->passthrough)
+                (*context->parser->passthrough) (context,
+                                                 context->partial_chunk->str,
+                                                 context->partial_chunk->len,
+                                                 context->user_data,
+                                                 &tmp_error);
+                  
               truncate_partial (context);
 
               if (tmp_error == NULL)
@@ -1737,6 +1705,26 @@ g_markup_parse_context_end_parse (GMarkupParseContext *context,
   return context->state != STATE_ERROR;
 }
 
+/**
+ * g_markup_parse_context_get_element:
+ * @context: a #GMarkupParseContext
+ * @returns: the name of the currently open element, or %NULL
+ *
+ * Retrieves the name of the currently open element.
+ *
+ * Since: 2.2
+ **/
+G_CONST_RETURN gchar *
+g_markup_parse_context_get_element (GMarkupParseContext *context)
+{
+  g_return_val_if_fail (context != NULL, NULL);
+
+  if (context->tag_stack == NULL) 
+    return NULL;
+  else
+    return current_element (context);
+} 
+
 /**
  * g_markup_parse_context_get_position:
  * @context: a #GMarkupParseContext
