@@ -40,6 +40,7 @@
 #include <config.h>
 
 #include "glib.h"
+#include "gprintfint.h"
 
 #include <string.h>
 #include <stdlib.h>
@@ -98,7 +99,8 @@ enum {
 
 static gboolean make_pipe            (gint                  p[2],
                                       GError              **error);
-static gboolean fork_exec_with_pipes (gboolean              dont_wait,
+static gboolean do_spawn_with_pipes  (gboolean              dont_wait,
+				      gboolean		    dont_return_handle,
 				      const gchar          *working_directory,
                                       gchar               **argv,
                                       gchar               **envp,
@@ -109,6 +111,7 @@ static gboolean fork_exec_with_pipes (gboolean              dont_wait,
                                       gboolean              child_inherits_stdin,
                                       GSpawnChildSetupFunc  child_setup,
                                       gpointer              user_data,
+                                      gint                 *child_pid,
                                       gint                 *standard_input,
                                       gint                 *standard_output,
                                       gint                 *standard_error,
@@ -124,22 +127,6 @@ g_spawn_error_quark (void)
   return quark;
 }
 
-/**
- * g_spawn_async:
- * @working_directory: child's current working directory, or %NULL to inherit parent's
- * @argv: child's argument vector
- * @envp: child's environment, or %NULL to inherit parent's
- * @flags: flags from #GSpawnFlags
- * @child_setup: function to run in the child just before <function>exec()</function>
- * @user_data: user data for @child_setup
- * @child_pid: return location for child process ID, or %NULL
- * @error: return location for error
- * 
- * See g_spawn_async_with_pipes() for a full description; this function
- * simply calls the g_spawn_async_with_pipes() without any pipes.
- * 
- * Return value: %TRUE on success, %FALSE if error is set
- **/
 gboolean
 g_spawn_async (const gchar          *working_directory,
                gchar               **argv,
@@ -171,8 +158,13 @@ close_and_invalidate (gint *fd)
 {
   gint ret;
 
-  ret = close (*fd);
-  *fd = -1;
+  if (*fd < 0)
+    return -1;
+  else
+    {
+      ret = close (*fd);
+      *fd = -1;
+    }
 
   return ret;
 }
@@ -190,7 +182,7 @@ read_data (GString     *str,
            GError     **error)
 {
   GIOStatus giostatus;
-  gint bytes;
+  gssize bytes;
   gchar buf[4096];
 
  again:
@@ -219,32 +211,6 @@ read_data (GString     *str,
     return READ_OK;
 }
 
-/**
- * g_spawn_sync:
- * @working_directory: child's current working directory, or %NULL to inherit parent's
- * @argv: child's argument vector
- * @envp: child's environment, or %NULL to inherit parent's
- * @flags: flags from #GSpawnFlags
- * @child_setup: function to run in the child just before <function>exec()</function>
- * @user_data: user data for @child_setup
- * @standard_output: return location for child output 
- * @standard_error: return location for child error messages
- * @exit_status: child exit status, as returned by <function>waitpid()</function>
- * @error: return location for error
- *
- * Executes a child synchronously (waits for the child to exit before returning).
- * All output from the child is stored in @standard_output and @standard_error,
- * if those parameters are non-%NULL. If @exit_status is non-%NULL, the exit status
- * of the child is stored there as it would be returned by <function>waitpid()</function>; standard UNIX
- * macros such as <function>WIFEXITED()</function> and <function>WEXITSTATUS()</function> must be used to evaluate the
- * exit status. If an error occurs, no data is returned in @standard_output,
- * @standard_error, or @exit_status.
- * 
- * This function calls g_spawn_async_with_pipes() internally; see that function
- * for full details on the other parameters.
- * 
- * Return value: %TRUE on success, %FALSE if an error was set.
- **/
 gboolean
 g_spawn_sync (const gchar          *working_directory,
               gchar               **argv,
@@ -259,6 +225,7 @@ g_spawn_sync (const gchar          *working_directory,
 {
   gint outpipe = -1;
   gint errpipe = -1;
+  gint pid;
   GIOChannel *outchannel = NULL;
   GIOChannel *errchannel = NULL;
   GPollFD outfd, errfd;
@@ -288,22 +255,24 @@ g_spawn_sync (const gchar          *working_directory,
   if (standard_error)
     *standard_error = NULL;
   
-  if (!fork_exec_with_pipes (FALSE,
-                             working_directory,
-                             argv,
-                             envp,
-                             !(flags & G_SPAWN_LEAVE_DESCRIPTORS_OPEN),
-                             (flags & G_SPAWN_SEARCH_PATH) != 0,
-                             (flags & G_SPAWN_STDOUT_TO_DEV_NULL) != 0,
-                             (flags & G_SPAWN_STDERR_TO_DEV_NULL) != 0,
-                             (flags & G_SPAWN_CHILD_INHERITS_STDIN) != 0,
-                             child_setup,
-                             user_data,
-                             NULL,
-                             standard_output ? &outpipe : NULL,
-                             standard_error ? &errpipe : NULL,
-			     &status,
-                             error))
+  if (!do_spawn_with_pipes (FALSE,
+			    TRUE,
+			    working_directory,
+			    argv,
+			    envp,
+			    !(flags & G_SPAWN_LEAVE_DESCRIPTORS_OPEN),
+			    (flags & G_SPAWN_SEARCH_PATH) != 0,
+			    (flags & G_SPAWN_STDOUT_TO_DEV_NULL) != 0,
+			    (flags & G_SPAWN_STDERR_TO_DEV_NULL) != 0,
+			    (flags & G_SPAWN_CHILD_INHERITS_STDIN) != 0,
+			    child_setup,
+			    user_data,
+			    &pid,
+			    NULL,
+			    standard_output ? &outpipe : NULL,
+			    standard_error ? &errpipe : NULL,
+			    &status,
+			    error))
     return FALSE;
 
   /* Read data from child. */
@@ -455,83 +424,6 @@ g_spawn_sync (const gchar          *working_directory,
     }
 }
 
-/**
- * g_spawn_async_with_pipes:
- * @working_directory: child's current working directory, or %NULL to inherit parent's
- * @argv: child's argument vector
- * @envp: child's environment, or %NULL to inherit parent's
- * @flags: flags from #GSpawnFlags
- * @child_setup: function to run in the child just before <function>exec()</function>
- * @user_data: user data for @child_setup
- * @child_pid: return location for child process ID, or %NULL
- * @standard_input: return location for file descriptor to write to child's stdin, or %NULL
- * @standard_output: return location for file descriptor to read child's stdout, or %NULL
- * @standard_error: return location for file descriptor to read child's stderr, or %NULL
- * @error: return location for error
- *
- * Executes a child program asynchronously (your program will not
- * block waiting for the child to exit). The child program is
- * specified by the only argument that must be provided, @argv. @argv
- * should be a %NULL-terminated array of strings, to be passed as the
- * argument vector for the child. The first string in @argv is of
- * course the name of the program to execute. By default, the name of
- * the program must be a full path; the <envar>PATH</envar> shell variable will only
- * be searched if you pass the %G_SPAWN_SEARCH_PATH flag.
- *
- * @envp is a %NULL-terminated array of strings, where each string
- * has the form <literal>KEY=VALUE</literal>. This will become
- * the child's environment. If @envp is %NULL, the child inherits its
- * parent's environment.
- *
- * @flags should be the bitwise OR of any flags you want to affect the
- * function's behavior. The %G_SPAWN_DO_NOT_REAP_CHILD means that the
- * child will not be automatically reaped; you must call 
- * <function>waitpid()</function> or handle %SIGCHLD yourself, or the 
- * child will become a zombie. %G_SPAWN_LEAVE_DESCRIPTORS_OPEN means 
- * that the parent's open file descriptors will be inherited by the child; 
- * otherwise all descriptors except stdin/stdout/stderr will be closed before
- * calling <function>exec()</function> in the child. %G_SPAWN_SEARCH_PATH 
- * means that <literal>argv[0]</literal> need not be an absolute path, it
- * will be looked for in the user's <envar>PATH</envar>. 
- * %G_SPAWN_STDOUT_TO_DEV_NULL means that the child's standard output 
- * will be discarded, instead of going to the same location as the parent's 
- * standard output. %G_SPAWN_STDERR_TO_DEV_NULL means that the child's 
- * standard error will be discarded. %G_SPAWN_CHILD_INHERITS_STDIN means that
- * the child will inherit the parent's standard input (by default,
- * the child's standard input is attached to /dev/null).
- *
- * @child_setup and @user_data are a function and user data to be
- * called in the child after GLib has performed all the setup it plans
- * to perform (including creating pipes, closing file descriptors,
- * etc.) but before calling <function>exec()</function>. That is, @child_setup 
- * is called just before calling <function>exec()</function> in the child. 
- * Obviously actions taken in this function will only affect the child, not 
- * the parent. 
- *
- * If non-%NULL, @child_pid will be filled with the child's process
- * ID. You can use the process ID to send signals to the child, or
- * to <function>waitpid()</function> if you specified the 
- * %G_SPAWN_DO_NOT_REAP_CHILD flag.
- *
- * If non-%NULL, the @standard_input, @standard_output, @standard_error
- * locations will be filled with file descriptors for writing to the child's
- * standard input or reading from its standard output or standard error.
- * The caller of g_spawn_async_with_pipes() must close these file descriptors
- * when they are no longer in use. If these parameters are %NULL, the
- * corresponding pipe won't be created.
- *
- * @error can be %NULL to ignore errors, or non-%NULL to report errors.
- * If an error is set, the function returns %FALSE. Errors
- * are reported even if they occur in the child (for example if the
- * executable in <literal>argv[0]</literal> is not found). Typically
- * the <literal>message</literal> field of returned errors should be displayed
- * to users. Possible errors are those from the #G_SPAWN_ERROR domain.
- *
- * If an error occurs, @child_pid, @standard_input, @standard_output,
- * and @standard_error will not be filled with valid values.
- * 
- * Return value: %TRUE on success, %FALSE if an error was set
- **/
 gboolean
 g_spawn_async_with_pipes (const gchar          *working_directory,
                           gchar               **argv,
@@ -554,43 +446,26 @@ g_spawn_async_with_pipes (const gchar          *working_directory,
   g_return_val_if_fail (standard_input == NULL ||
                         !(flags & G_SPAWN_CHILD_INHERITS_STDIN), FALSE);
   
-  return fork_exec_with_pipes (!(flags & G_SPAWN_DO_NOT_REAP_CHILD),
-                               working_directory,
-                               argv,
-                               envp,
-                               !(flags & G_SPAWN_LEAVE_DESCRIPTORS_OPEN),
-                               (flags & G_SPAWN_SEARCH_PATH) != 0,
-                               (flags & G_SPAWN_STDOUT_TO_DEV_NULL) != 0,
-                               (flags & G_SPAWN_STDERR_TO_DEV_NULL) != 0,
-                               (flags & G_SPAWN_CHILD_INHERITS_STDIN) != 0,
-                               child_setup,
-                               user_data,
-                               standard_input,
-                               standard_output,
-                               standard_error,
-			       NULL,
-                               error);
+  return do_spawn_with_pipes (TRUE,
+			      !(flags & G_SPAWN_DO_NOT_REAP_CHILD),
+			      working_directory,
+			      argv,
+			      envp,
+			      !(flags & G_SPAWN_LEAVE_DESCRIPTORS_OPEN),
+			      (flags & G_SPAWN_SEARCH_PATH) != 0,
+			      (flags & G_SPAWN_STDOUT_TO_DEV_NULL) != 0,
+			      (flags & G_SPAWN_STDERR_TO_DEV_NULL) != 0,
+			      (flags & G_SPAWN_CHILD_INHERITS_STDIN) != 0,
+			      child_setup,
+			      user_data,
+			      child_pid,
+			      standard_input,
+			      standard_output,
+			      standard_error,
+			      NULL,
+			      error);
 }
 
-/**
- * g_spawn_command_line_sync:
- * @command_line: a command line 
- * @standard_output: return location for child output
- * @standard_error: return location for child errors
- * @exit_status: return location for child exit status
- * @error: return location for errors
- *
- * A simple version of g_spawn_sync() with little-used parameters
- * removed, taking a command line instead of an argument vector.  See
- * g_spawn_sync() for full details. @command_line will be parsed by
- * g_shell_parse_argv(). Unlike g_spawn_sync(), the %G_SPAWN_SEARCH_PATH flag
- * is enabled. Note that %G_SPAWN_SEARCH_PATH can have security
- * implications, so consider using g_spawn_sync() directly if
- * appropriate. Possible errors are those from g_spawn_sync() and those
- * from g_shell_parse_argv().
- * 
- * Return value: %TRUE on success, %FALSE if an error was set
- **/
 gboolean
 g_spawn_command_line_sync (const gchar  *command_line,
                            gchar       **standard_output,
@@ -623,21 +498,6 @@ g_spawn_command_line_sync (const gchar  *command_line,
   return retval;
 }
 
-/**
- * g_spawn_command_line_async:
- * @command_line: a command line
- * @error: return location for errors
- * 
- * A simple version of g_spawn_async() that parses a command line with
- * g_shell_parse_argv() and passes it to g_spawn_async(). Runs a
- * command line in the background. Unlike g_spawn_async(), the
- * %G_SPAWN_SEARCH_PATH flag is enabled, other flags are not. Note
- * that %G_SPAWN_SEARCH_PATH can have security implications, so
- * consider using g_spawn_async() directly if appropriate. Possible
- * errors are those from g_shell_parse_argv() and g_spawn_async().
- * 
- * Return value: %TRUE on success, %FALSE if error is set.
- **/
 gboolean
 g_spawn_command_line_async (const gchar *command_line,
                             GError     **error)
@@ -666,25 +526,26 @@ g_spawn_command_line_async (const gchar *command_line,
 }
 
 static gint
-do_exec (gboolean              dont_wait,
-	 gint                  child_err_report_fd,
-         gint                  stdin_fd,
-         gint                  stdout_fd,
-         gint                  stderr_fd,
-         const gchar          *working_directory,
-         gchar               **argv,
-         gchar               **envp,
-         gboolean              close_descriptors,
-         gboolean              search_path,
-         gboolean              stdout_to_null,
-         gboolean              stderr_to_null,
-         gboolean              child_inherits_stdin,
-         GSpawnChildSetupFunc  child_setup,
-         gpointer              user_data)
+do_spawn (gboolean              dont_wait,
+	  gint                  child_err_report_fd,
+	  gint                  stdin_fd,
+	  gint                  stdout_fd,
+	  gint                  stderr_fd,
+	  const gchar          *working_directory,
+	  gchar               **argv,
+	  gchar               **envp,
+	  gboolean              close_descriptors,
+	  gboolean              search_path,
+	  gboolean              stdout_to_null,
+	  gboolean              stderr_to_null,
+	  gboolean              child_inherits_stdin,
+	  GSpawnChildSetupFunc  child_setup,
+	  gpointer              user_data)
 {
   gchar **new_argv;
   gchar args[ARG_COUNT][10];
   gint i;
+  int rc;
   int argc = 0;
 
   SETUP_DEBUG();
@@ -695,12 +556,12 @@ do_exec (gboolean              dont_wait,
   new_argv = g_new (gchar *, argc + 1 + ARG_COUNT);
 
   new_argv[0] = "gspawn-win32-helper";
-  sprintf (args[ARG_CHILD_ERR_REPORT], "%d", child_err_report_fd);
+  _g_sprintf (args[ARG_CHILD_ERR_REPORT], "%d", child_err_report_fd);
   new_argv[ARG_CHILD_ERR_REPORT] = args[ARG_CHILD_ERR_REPORT];
 
   if (stdin_fd >= 0)
     {
-      sprintf (args[ARG_STDIN], "%d", stdin_fd);
+      _g_sprintf (args[ARG_STDIN], "%d", stdin_fd);
       new_argv[ARG_STDIN] = args[ARG_STDIN];
     }
   else if (child_inherits_stdin)
@@ -716,7 +577,7 @@ do_exec (gboolean              dont_wait,
 
   if (stdout_fd >= 0)
     {
-      sprintf (args[ARG_STDOUT], "%d", stdout_fd);
+      _g_sprintf (args[ARG_STDOUT], "%d", stdout_fd);
       new_argv[ARG_STDOUT] = args[ARG_STDOUT];
     }
   else if (stdout_to_null)
@@ -730,7 +591,7 @@ do_exec (gboolean              dont_wait,
 
   if (stderr_fd >= 0)
     {
-      sprintf (args[ARG_STDERR], "%d", stderr_fd);
+      _g_sprintf (args[ARG_STDERR], "%d", stderr_fd);
       new_argv[ARG_STDERR] = args[ARG_STDERR];
     }
   else if (stderr_to_null)
@@ -743,7 +604,7 @@ do_exec (gboolean              dont_wait,
     }
 
   if (working_directory && *working_directory)
-    new_argv[ARG_WORKING_DIRECTORY] = working_directory;
+    new_argv[ARG_WORKING_DIRECTORY] = g_strdup (working_directory);
   else
     new_argv[ARG_WORKING_DIRECTORY] = "-";
 
@@ -786,11 +647,9 @@ do_exec (gboolean              dont_wait,
     /* Let's hope envp hasn't mucked with PATH so that
      * gspawn-win32-helper.exe isn't found.
      */
-    spawnvpe (P_NOWAIT, "gspawn-win32-helper", new_argv, envp);
+    rc = spawnvpe (P_NOWAIT, "gspawn-win32-helper", new_argv, envp);
   else
-    spawnvp (P_NOWAIT, "gspawn-win32-helper", new_argv);
-
-  /* FIXME: What if gspawn-win32-helper.exe isn't found? */
+    rc = spawnvp (P_NOWAIT, "gspawn-win32-helper", new_argv);
 
   /* Close the child_err_report_fd and the other process's ends of the
    * pipes in this process, otherwise the reader will never get
@@ -804,9 +663,10 @@ do_exec (gboolean              dont_wait,
   if (stderr_fd >= 0)
     close (stderr_fd);
 
+  g_free (new_argv[ARG_WORKING_DIRECTORY]);
   g_free (new_argv);
 
-  return 0;
+  return rc;
 }
 
 static gboolean
@@ -857,28 +717,30 @@ read_ints (int      fd,
 }
 
 static gboolean
-fork_exec_with_pipes (gboolean              dont_wait,
-                      const gchar          *working_directory,
-                      gchar               **argv,
-                      gchar               **envp,
-                      gboolean              close_descriptors,
-                      gboolean              search_path,
-                      gboolean              stdout_to_null,
-                      gboolean              stderr_to_null,
-		      gboolean              child_inherits_stdin,
-                      GSpawnChildSetupFunc  child_setup,
-                      gpointer              user_data,
-                      gint                 *standard_input,
-                      gint                 *standard_output,
-                      gint                 *standard_error,
-		      gint                 *exit_status,
-                      GError              **error)     
+do_spawn_with_pipes (gboolean              dont_wait,
+		     gboolean		   dont_return_handle,
+		     const gchar          *working_directory,
+		     gchar               **argv,
+		     gchar               **envp,
+		     gboolean              close_descriptors,
+		     gboolean              search_path,
+		     gboolean              stdout_to_null,
+		     gboolean              stderr_to_null,
+		     gboolean              child_inherits_stdin,
+		     GSpawnChildSetupFunc  child_setup,
+		     gpointer              user_data,
+		     gint                 *child_pid,
+		     gint                 *standard_input,
+		     gint                 *standard_output,
+		     gint                 *standard_error,
+		     gint                 *exit_status,
+		     GError              **error)     
 {
   gint stdin_pipe[2] = { -1, -1 };
   gint stdout_pipe[2] = { -1, -1 };
   gint stderr_pipe[2] = { -1, -1 };
   gint child_err_report_pipe[2] = { -1, -1 };
-  gint status;
+  gint helper = -1;
   gint buf[2];
   gint n_ints = 0;
   
@@ -894,53 +756,73 @@ fork_exec_with_pipes (gboolean              dont_wait,
   if (standard_error && !make_pipe (stderr_pipe, error))
     goto cleanup_and_fail;
 
-  status = do_exec (dont_wait,
-		    child_err_report_pipe[1],
-		    stdin_pipe[0],
-		    stdout_pipe[1],
-		    stderr_pipe[1],
-		    working_directory,
-		    argv,
-		    envp,
-		    close_descriptors,
-		    search_path,
-		    stdout_to_null,
-		    stderr_to_null,
-		    child_inherits_stdin,
-		    child_setup,
-		    user_data);
+  helper = do_spawn (dont_wait,
+		     child_err_report_pipe[1],
+		     stdin_pipe[0],
+		     stdout_pipe[1],
+		     stderr_pipe[1],
+		     working_directory,
+		     argv,
+		     envp,
+		     close_descriptors,
+		     search_path,
+		     stdout_to_null,
+		     stderr_to_null,
+		     child_inherits_stdin,
+		     child_setup,
+		     user_data);
       
+  /* do_spawn() returns -1 if gspawn-win32-helper couldn't be run */
+  if (helper == -1)
+    {
+      g_set_error (error,
+		   G_SPAWN_ERROR,
+		   G_SPAWN_ERROR_FAILED,
+		   _("Failed to execute helper program"));
+      goto cleanup_and_fail;
+    }
+
   if (!read_ints (child_err_report_pipe[0],
 		  buf, 2, &n_ints,
-		  error))
+		  error) ||
+      n_ints != 2)
     goto cleanup_and_fail;
         
-  if (n_ints == 2)
+  /* Error code from gspawn-win32-helper. */
+  switch (buf[0])
     {
-      /* Error from the child. */
-      
-      switch (buf[0])
+    case CHILD_NO_ERROR:
+      if (child_pid && dont_wait && !dont_return_handle)
 	{
-	case CHILD_NO_ERROR:
-	  break;
-	  
-	case CHILD_CHDIR_FAILED:
-	  g_set_error (error,
-		       G_SPAWN_ERROR,
-		       G_SPAWN_ERROR_CHDIR,
-		       _("Failed to change to directory '%s' (%s)"),
-		       working_directory,
-		       g_strerror (buf[1]));
-	  goto cleanup_and_fail;
-	  
-	case CHILD_SPAWN_FAILED:
-	  g_set_error (error,
-		       G_SPAWN_ERROR,
-		       G_SPAWN_ERROR_FAILED,
-		       _("Failed to execute child process (%s)"),
-		       g_strerror (buf[1]));
-	  goto cleanup_and_fail;
+	  /* helper is our HANDLE for gspawn-win32-helper. It has
+	   * told us the HANDLE of its child. Duplicate that into
+	   * a HANDLE valid in this process.
+	   */
+	  if (!DuplicateHandle ((HANDLE) helper, (HANDLE) buf[1],
+				GetCurrentProcess (), (LPHANDLE) child_pid,
+				0, TRUE, DUPLICATE_SAME_ACCESS))
+	    *child_pid = 0;
 	}
+      else if (child_pid)
+	*child_pid = 0;
+      break;
+      
+    case CHILD_CHDIR_FAILED:
+      g_set_error (error,
+		   G_SPAWN_ERROR,
+		   G_SPAWN_ERROR_CHDIR,
+		   _("Failed to change to directory '%s' (%s)"),
+		   working_directory,
+		   g_strerror (buf[1]));
+      goto cleanup_and_fail;
+      
+    case CHILD_SPAWN_FAILED:
+      g_set_error (error,
+		   G_SPAWN_ERROR,
+		   G_SPAWN_ERROR_FAILED,
+		   _("Failed to execute child process (%s)"),
+		   g_strerror (buf[1]));
+      goto cleanup_and_fail;
     }
 
   /* Success against all odds! return the information */
@@ -952,11 +834,14 @@ fork_exec_with_pipes (gboolean              dont_wait,
   if (standard_error)
     *standard_error = stderr_pipe[0];
   if (exit_status)
-    *exit_status = status;
+    *exit_status = buf[1];
+  CloseHandle ((HANDLE) helper);
   
   return TRUE;
 
  cleanup_and_fail:
+  if (helper != -1)
+    CloseHandle ((HANDLE) helper);
   close_and_invalidate (&child_err_report_pipe[0]);
   close_and_invalidate (&child_err_report_pipe[1]);
   close_and_invalidate (&stdin_pipe[0]);
