@@ -45,6 +45,7 @@
 #include <signal.h>
 #endif
 #include "glib.h"
+#include "gprintfint.h"
 
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -183,7 +184,7 @@ g_strdup_vprintf (const gchar *format,
 {
   gchar *buffer;
 #ifdef HAVE_VASPRINTF
-  if (vasprintf (&buffer, format, args1) < 0)
+  if (_g_vasprintf (&buffer, format, args1) < 0)
     buffer = NULL;
   else if (!g_mem_is_system_malloc ()) 
     {
@@ -198,7 +199,7 @@ g_strdup_vprintf (const gchar *format,
 
   buffer = g_new (gchar, g_printf_string_upper_bound (format, args1));
 
-  vsprintf (buffer, format, args2);
+  _g_vsprintf (buffer, format, args2);
   va_end (args2);
 #endif
   return buffer;
@@ -544,7 +545,7 @@ g_ascii_formatd (gchar       *buffer,
     return NULL;
 
       
-  g_snprintf (buffer, buf_len, format, d);
+  _g_snprintf (buffer, buf_len, format, d);
 
   locale_data = localeconv ();
   decimal_point = locale_data->decimal_point;
@@ -580,6 +581,159 @@ g_ascii_formatd (gchar       *buffer,
   return buffer;
 }
 
+/**
+ * g_ascii_strtoull:
+ * @nptr:    the string to convert to a numeric value.
+ * @endptr:  if non-%NULL, it returns the character after
+ *           the last character used in the conversion.
+ * @base:    to be used for the conversion, 2..36 or 0
+ *
+ * Converts a string to a #guint64 value.
+ * This function behaves like the standard strtoull() function
+ * does in the C locale. It does this without actually
+ * changing the current locale, since that would not be
+ * thread-safe.
+ *
+ * This function is typically used when reading configuration
+ * files or other non-user input that should be locale independent.
+ * To handle input from the user you should normally use the
+ * locale-sensitive system strtoull() function.
+ *
+ * If the correct value would cause overflow, %G_MAXUINT64
+ * is returned, and %ERANGE is stored in %errno.
+ *
+ * Return value: the #guint64 value.
+ *
+ * Since: 2.2
+ **/
+guint64
+g_ascii_strtoull (const gchar *nptr,
+		  gchar      **endptr,
+		  guint        base)
+{
+  /* this code is based on on the strtol(3) code from GNU libc released under
+   * the GNU Lesser General Public License.
+   *
+   * Copyright (C) 1991,92,94,95,96,97,98,99,2000,01,02
+   *        Free Software Foundation, Inc.
+   */
+#define ISSPACE(c)		((c) == ' ' || (c) == '\f' || (c) == '\n' || \
+				 (c) == '\r' || (c) == '\t' || (c) == '\v')
+#define ISUPPER(c)		((c) >= 'A' && (c) <= 'Z')
+#define ISLOWER(c)		((c) >= 'a' && (c) <= 'z')
+#define ISALPHA(c)		(ISUPPER (c) || ISLOWER (c))
+#define	TOUPPER(c)		(ISLOWER (c) ? (c) - 'a' + 'A' : (c))
+#define	TOLOWER(c)		(ISUPPER (c) ? (c) - 'A' + 'a' : (c))
+  gboolean negative, overflow;
+  guint64 cutoff;
+  guint64 cutlim;
+  guint64 ui64;
+  const gchar *s, *save;
+  guchar c;
+  
+  g_return_val_if_fail (nptr != NULL, 0);
+  
+  if (base == 1 || base > 36)
+    {
+      errno = EINVAL;
+      return 0;
+    }
+  
+  save = s = nptr;
+  
+  /* Skip white space.  */
+  while (ISSPACE (*s))
+    ++s;
+  if (!*s)
+    goto noconv;
+  
+  /* Check for a sign.  */
+  negative = FALSE;
+  if (*s == '-')
+    {
+      negative = TRUE;
+      ++s;
+    }
+  else if (*s == '+')
+    ++s;
+  
+  /* Recognize number prefix and if BASE is zero, figure it out ourselves.  */
+  if (*s == '0')
+    {
+      if ((base == 0 || base == 16) && TOUPPER (s[1]) == 'X')
+	{
+	  s += 2;
+	  base = 16;
+	}
+      else if (base == 0)
+	base = 8;
+    }
+  else if (base == 0)
+    base = 10;
+  
+  /* Save the pointer so we can check later if anything happened.  */
+  save = s;
+  cutoff = G_MAXUINT64 / base;
+  cutlim = G_MAXUINT64 % base;
+  
+  overflow = FALSE;
+  ui64 = 0;
+  c = *s;
+  for (; c; c = *++s)
+    {
+      if (c >= '0' && c <= '9')
+	c -= '0';
+      else if (ISALPHA (c))
+	c = TOUPPER (c) - 'A' + 10;
+      else
+	break;
+      if (c >= base)
+	break;
+      /* Check for overflow.  */
+      if (ui64 > cutoff || (ui64 == cutoff && c > cutlim))
+	overflow = TRUE;
+      else
+	{
+	  ui64 *= base;
+	  ui64 += c;
+	}
+    }
+  
+  /* Check if anything actually happened.  */
+  if (s == save)
+    goto noconv;
+  
+  /* Store in ENDPTR the address of one character
+     past the last character we converted.  */
+  if (endptr)
+    *endptr = (gchar*) s;
+  
+  if (overflow)
+    {
+      errno = ERANGE;
+      return G_MAXUINT64;
+    }
+  
+  /* Return the result of the appropriate sign.  */
+  return negative ? -ui64 : ui64;
+  
+ noconv:
+  /* We must handle a special case here: the base is 0 or 16 and the
+     first two characters are '0' and 'x', but the rest are no
+     hexadecimal digits.  This is no error case.  We return 0 and
+     ENDPTR points to the `x`.  */
+  if (endptr)
+    {
+      if (save - nptr >= 2 && TOUPPER (save[-1]) == 'X'
+	  && save[-2] == '0')
+	*endptr = (gchar*) &save[-1];
+      else
+	/*  There was no number to convert.  */
+	*endptr = (gchar*) nptr;
+    }
+  return 0;
+}
+
 
 G_CONST_RETURN gchar*
 g_strerror (gint errnum)
@@ -1036,7 +1190,7 @@ g_strerror (gint errnum)
       g_static_private_set (&msg_private, msg, g_free);
     }
 
-  sprintf (msg, "unknown error (%d)", errnum);
+  _g_sprintf (msg, "unknown error (%d)", errnum);
 
   return msg;
 }
@@ -1185,7 +1339,7 @@ extern const char *strsignal(int);
       g_static_private_set (&msg_private, msg, g_free);
     }
 
-  sprintf (msg, "unknown signal (%d)", signum);
+  _g_sprintf (msg, "unknown signal (%d)", signum);
   
   return msg;
 }
@@ -1377,6 +1531,18 @@ g_ascii_strup (const gchar *str,
   return result;
 }
 
+/**
+ * g_strdown:
+ * @string: the string to convert.
+ * 
+ * Converts a string to lower case.  
+ * 
+ * Return value: the string 
+ *
+ * Deprecated: This function is totally broken for the reasons discussed in 
+ * the g_strncasecmp() docs - use g_ascii_strdown() or g_utf8_strdown() 
+ * instead.
+ **/
 gchar*
 g_strdown (gchar *string)
 {
@@ -1396,6 +1562,17 @@ g_strdown (gchar *string)
   return (gchar *) string;
 }
 
+/**
+ * g_strup:
+ * @string: the string to convert.
+ * 
+ * Converts a string to upper case. 
+ * 
+ * Return value: the string
+ *
+ * Deprecated: This function is totally broken for the reasons discussed in 
+ * the g_strncasecmp() docs - use g_ascii_strup() or g_utf8_strup() instead.
+ **/
 gchar*
 g_strup (gchar *string)
 {
@@ -1612,6 +1789,20 @@ g_ascii_strncasecmp (const gchar *s1,
     return 0;
 }
 
+/**
+ * g_strcasecmp:
+ * @s1: a string.
+ * @s2: a string to compare with @s1.
+ * 
+ * A case-insensitive string comparison, corresponding to the standard
+ * strcasecmp() function on platforms which support it.
+ *
+ * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2, 
+ *   or a positive value if @s1 &gt; @s2.
+ *
+ * Deprecated: See g_strncasecmp() for a discussion of why this function is 
+ *   deprecated and how to replace it.
+ **/
 gint
 g_strcasecmp (const gchar *s1,
 	      const gchar *s2)
@@ -1643,6 +1834,33 @@ g_strcasecmp (const gchar *s1,
 #endif
 }
 
+/**
+ * g_strncasecmp:
+ * @s1: a string.
+ * @s2: a string to compare with @s1.
+ * @n: the maximum number of characters to compare.
+ * 
+ * A case-insensitive string comparison, corresponding to the standard
+ * strncasecmp() function on platforms which support it.
+ * It is similar to g_strcasecmp() except it only compares the first @n 
+ * characters of the strings.
+ * 
+ * Return value: 0 if the strings match, a negative value if @s1 &lt; @s2, 
+ *   or a positive value if @s1 &gt; @s2.
+ *
+ * Deprecated: The problem with g_strncasecmp() is that it does the 
+ * comparison by calling toupper()/tolower(). These functions are
+ * locale-specific and operate on single bytes. However, it is impossible
+ * to handle things correctly from an I18N standpoint by operating on
+ * bytes, since characters may be multibyte. Thus g_strncasecmp() is
+ * broken if your string is guaranteed to be ASCII, since it's
+ * locale-sensitive, and it's broken if your string is localized, since
+ * it doesn't work on many encodings at all, including UTF-8, EUC-JP,
+ * etc.
+ * There are therefore two replacement functions: g_ascii_strncasecmp(),
+ * which only works on ASCII and is not locale-sensitive, and
+ * g_utf8_casefold(), which is good for case-insensitive sorting of UTF-8.
+ **/
 gint
 g_strncasecmp (const gchar *s1,
 	       const gchar *s2,
@@ -1871,16 +2089,18 @@ g_strchug (gchar *string)
 gchar*
 g_strchomp (gchar *string)
 {
-  gchar *s;
+  gsize len;
 
   g_return_val_if_fail (string != NULL, NULL);
 
-  if (!*string)
-    return string;
-
-  for (s = string + strlen (string) - 1; s >= string && g_ascii_isspace ((guchar)*s); 
-       s--)
-    *s = '\0';
+  len = strlen (string);
+  while (len--)
+    {
+      if (g_ascii_isspace ((guchar) string[len]))
+	string[len] = '\0';
+      else
+	break;
+    }
 
   return string;
 }
@@ -2271,3 +2491,63 @@ g_strrstr_len (const gchar *haystack,
 }
 
 
+/**
+ * g_str_has_suffix:
+ * @str: a nul-terminated string.
+ * @suffix: the nul-terminated suffix to look for.
+ *
+ * Looks whether the string @str ends with @suffix.
+ *
+ * Return value: %TRUE if @str end with @suffix, %FALSE otherwise.
+ *
+ * Since: 2.2
+ **/
+gboolean
+g_str_has_suffix (const gchar  *str,
+		  const gchar  *suffix)
+{
+  int str_len;
+  int suffix_len;
+  
+  g_return_val_if_fail (str != NULL, FALSE);
+  g_return_val_if_fail (suffix != NULL, FALSE);
+
+  str_len = strlen (str);
+  suffix_len = strlen (suffix);
+
+  if (str_len < suffix_len)
+    return FALSE;
+
+  return strcmp (str + str_len - suffix_len, suffix) == 0;
+}
+
+/**
+ * g_str_has_prefix:
+ * @str: a nul-terminated string.
+ * @prefix: the nul-terminated prefix to look for.
+ *
+ * Looks whether the string @str begins with @prefix.
+ *
+ * Return value: %TRUE if @str begins with @prefix, %FALSE otherwise.
+ *
+ * Since: 2.2
+ **/
+gboolean
+g_str_has_prefix (const gchar  *str,
+		  const gchar  *prefix)
+{
+  int str_len;
+  int prefix_len;
+  
+  g_return_val_if_fail (str != NULL, FALSE);
+  g_return_val_if_fail (prefix != NULL, FALSE);
+
+  str_len = strlen (str);
+  prefix_len = strlen (prefix);
+
+  if (str_len < prefix_len)
+    return FALSE;
+  
+  return strncmp (str, prefix, prefix_len) == 0;
+}
+
