@@ -53,6 +53,7 @@
 #define	G_IMPLEMENT_INLINES 1
 #define	__G_UTILS_C__
 #include "glib.h"
+#include "gprintfint.h"
 
 #ifdef	MAXPATHLEN
 #define	G_PATH_LENGTH	MAXPATHLEN
@@ -80,6 +81,10 @@
 #include <langinfo.h>
 #endif
 
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+#include <libintl.h>
+#endif
+
 const guint glib_major_version = GLIB_MAJOR_VERSION;
 const guint glib_minor_version = GLIB_MINOR_VERSION;
 const guint glib_micro_version = GLIB_MICRO_VERSION;
@@ -336,54 +341,6 @@ g_find_program_in_path (const gchar *program)
   return NULL;
 }
 
-gint
-g_snprintf (gchar	*str,
-	    gulong	 n,
-	    gchar const *fmt,
-	    ...)
-{
-  va_list args;
-  gint retval;
-
-  va_start (args, fmt);
-  retval = g_vsnprintf (str, n, fmt, args);
-  va_end (args);
-  
-  return retval;
-}
-
-gint
-g_vsnprintf (gchar	 *str,
-	     gulong	  n,
-	     gchar const *fmt,
-	     va_list      args)
-{
-#ifdef	HAVE_VSNPRINTF_C99
-  g_return_val_if_fail (n == 0 || str != NULL, 0);
-  g_return_val_if_fail (fmt != NULL, 0);
-
-  return vsnprintf (str, n, fmt, args);
-#else	/* !HAVE_VSNPRINTF_C99 */
-  gchar *printed;
-  gint retval;
-
-  g_return_val_if_fail (n == 0 || str != NULL, 0);
-  g_return_val_if_fail (fmt != NULL, 0);
-
-  printed = g_strdup_vprintf (fmt, args);
-  retval = strlen (printed);
-  if (n > 0)
-    {
-      strncpy (str, printed, n - 1);
-      str[n-1] = '\0';
-    }
-
-  g_free (printed);
-  
-  return retval;
-#endif /* !HAVE_VSNPRINTF_C99 */
-}
-
 guint	     
 g_parse_debug_string  (const gchar     *string, 
 		       const GDebugKey *keys, 
@@ -426,6 +383,17 @@ g_parse_debug_string  (const gchar     *string,
   return result;
 }
 
+/**
+ * g_basename:
+ * @file_name: the name of the file.
+ * 
+ * Gets the name of the file without any leading directory components.  
+ * It returns a pointer into the given file name string.
+ * 
+ * Return value: the name of the file without any leading directory components.
+ *
+ * Deprecated: Use g_path_get_basename() instead. 
+ **/
 G_CONST_RETURN gchar*
 g_basename (const gchar	   *file_name)
 {
@@ -744,10 +712,12 @@ g_get_any_init (void)
 #endif /* G_OS_WIN32 */
 	}
       
-      if (!g_home_dir)
-	g_home_dir = g_strdup (g_getenv ("HOME"));
-      
 #ifdef G_OS_WIN32
+      /* We check $HOME first for Win32, though it is a last resort for Unix
+       * where we prefer the results of getpwuid().
+       */
+      g_home_dir = g_strdup (g_getenv ("HOME"));
+      
       /* In case HOME is Unix-style (it happens), convert it to
        * Windows style.
        */
@@ -879,8 +849,11 @@ g_get_any_init (void)
 	  }
       }
 #  endif /* G_OS_WIN32 */
-      
+
 #endif /* !HAVE_PWD_H */
+
+      if (!g_home_dir)
+	g_home_dir = g_strdup (g_getenv ("HOME"));
       
 #ifdef __EMX__
       /* change '\\' in %HOME% to '/' */
@@ -988,6 +961,72 @@ g_set_prgname (const gchar *prgname)
   G_UNLOCK (g_prgname);
 }
 
+G_LOCK_DEFINE (g_application_name);
+static gchar *g_application_name = NULL;
+
+/**
+ * g_get_application_name:
+ * 
+ * Gets a human-readable name for the application, as set by
+ * g_set_application_name(). This name should be localized if
+ * possible, and is intended for display to the user.  Contrast with
+ * g_get_prgname(), which gets a non-localized name. If
+ * g_set_application_name() has not been called, returns the result of
+ * g_get_prgname() (which may be %NULL if g_set_prgname() has also not
+ * been called).
+ * 
+ * Return value: human-readable application name. may return %NULL
+ *
+ * Since: 2.2
+ **/
+G_CONST_RETURN gchar*
+g_get_application_name (void)
+{
+  gchar* retval;
+
+  G_LOCK (g_application_name);
+  retval = g_application_name;
+  G_UNLOCK (g_application_name);
+
+  if (retval == NULL)
+    return g_get_prgname ();
+  
+  return retval;
+}
+
+/**
+ * g_set_application_name:
+ * @application_name: localized name of the application
+ *
+ * Sets a human-readable name for the application. This name should be
+ * localized if possible, and is intended for display to the user.
+ * Contrast with g_set_prgname(), which sets a non-localized name.
+ * g_set_prgname() will be called automatically by gtk_init(),
+ * but g_set_application_name() will not.
+ *
+ * Note that for thread safety reasons, this function can only
+ * be called once.
+ *
+ * The application name will be used in contexts such as error messages,
+ * or when displaying an application's name in the task list.
+ * 
+ **/
+void
+g_set_application_name (const gchar *application_name)
+{
+  gboolean already_set = FALSE;
+	
+  G_LOCK (g_application_name);
+  if (g_application_name)
+    already_set = TRUE;
+  else
+    g_application_name = g_strdup (application_name);
+  G_UNLOCK (g_application_name);
+
+  if (already_set)
+    g_warning ("g_set_application() name called multiple times");
+}
+
 guint
 g_direct_hash (gconstpointer v)
 {
@@ -1085,6 +1124,9 @@ _glib_gettext (const gchar *str)
   if (!_glib_gettext_initialized)
     {
       bindtextdomain(GETTEXT_PACKAGE, GLIB_LOCALE_DIR);
+#    ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+      bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#    endif
       _glib_gettext_initialized = TRUE;
     }
   
